/*
 *  CPE 471 lab 4 - draws a box without sides
 *  glut/OpenGL application 
 *  uses glm for matrices
 *  Has support for simple transforms - check the keyboard function
 *
 *  Created by zwood on 1/6/12 
 *  Copyright 2010 Cal Poly. All rights reserved.
 *
 *****************************************************************************/
#ifdef __APPLE__
#include "GLUT/glut.h"
#include <OPENGL/gl.h>
#endif
#ifdef __unix__
#include <GL/glut.h>
#endif

#include "ObjMeshLoaderSimple.h"
#include <stdlib.h>
#include <stdio.h>
#include <fstream>
#include "mesh.h"
#include "GLSL_helper.h"
#include "glm/glm.hpp"
#include "glm/gtc/matrix_transform.hpp" //perspective, trans etc
#include "glm/gtc/type_ptr.hpp" //value_ptr
#include <math.h>


class Particle {
   public:
   float mass;
   float position[3];
   float velocity[3];
   float force[3];
   
   Particle();
   Particle(float inputMass);
   Particle(float inputMass, float xPos, float yPos, float zPos);
};

Particle::Particle() {
   for(int index = 0; index < 3; index++) {
      position[index] = 0;
      velocity[index] = 0;
      force[index] = 0;
   }
}

Particle::Particle(float inputMass) {
   mass = inputMass;
   for(int index = 0; index < 3; index++) {
      position[index] = 0;
      velocity[index] = 0;
      force[index] = 0;
   }
}

Particle::Particle(float inputMass, float xPos, float yPos, float zPos) {
   mass = inputMass;

   position[0] = xPos;
   position[1] = yPos;
   position[2] = zPos;

   for(int index = 0; index < 3; index++) {
      velocity[index] = 0;
      force[index] = 0;
   }
}


class ParticleSystem {
   public:
   int ParticleGetState();
   int ParticleSetState();
   void Clear_Forces();
   void Compute_Forces();
   int ParticleDerivative();
   void ScaleVector(float deltaT);
   void AddVectors();
   void EulerStep(float deltaT);
   int ParticleDims();
   void printParticle();

   ParticleSystem(int size, float mass, float springInput, float dampInput);

   float temp1[12];
   float temp2[12];
   Particle particle[2];
   int numP;
   float time;
   float springConstant;
   float dampConstant;
};

ParticleSystem::ParticleSystem(int size, float mass, float springInput, float dampInput) {
   numP = size;
   time = 0;
   springConstant = springInput;
   dampConstant = dampInput;

   particle[0] = Particle(mass, 0, -3, 0);
   particle[1] = Particle(mass, 0, 8, 0);

   printParticle();
}

void ParticleSystem::printParticle() {
   for(int index = 0; index < 2; index++) {
      printf("%d 0: position: %.3f, velocity: %.3f, force: %.3f, mass: %.1f\n", index, particle[index].position[0], particle[index].velocity[0], particle[index].force[0], particle[index].mass); 
      printf("%d 1: position: %.3f, velocity: %.3f, force: %.3f, mass: %.1f\n", index, particle[index].position[1], particle[index].velocity[1], particle[index].force[1], particle[index].mass); 
      printf("%d 2: position: %.3f, velocity: %.3f, force: %.3f, mass: %.1f\n", index, particle[index].position[2], particle[index].velocity[2], particle[index].force[2], particle[index].mass); 
   }
   printf("time : %.1f\n", time);
}

int ParticleSystem::ParticleDims() {
   return (6 * numP);
}

int ParticleSystem::ParticleGetState() {
   int index, destIndex = 0;
   for(index = 0; index < numP; index++) {
      temp2[destIndex++] = particle[index].position[0];
      temp2[destIndex++] = particle[index].position[1];
      temp2[destIndex++] = particle[index].position[2];
      temp2[destIndex++] = particle[index].velocity[0];
      temp2[destIndex++] = particle[index].velocity[1];
      temp2[destIndex++] = particle[index].velocity[2];
   }
}

int ParticleSystem::ParticleSetState() {
   int index, sourceIndex = 0;
   for(index = 0; index < numP; index++) {
      particle[index].position[0] = temp2[sourceIndex++];
      particle[index].position[1] = temp2[sourceIndex++];
      particle[index].position[2] = temp2[sourceIndex++];
      particle[index].velocity[0] = temp2[sourceIndex++];
      particle[index].velocity[1] = temp2[sourceIndex++];
      particle[index].velocity[2] = temp2[sourceIndex++];
   }
}

void ParticleSystem::Clear_Forces() {
   int index;
   for(index = 0; index < numP; index++) {
      particle[index].force[0] = 0;
      particle[index].force[1] = 0;
      particle[index].force[2] = 0;
   }
}

void ParticleSystem::Compute_Forces() {
   float xPosVec = (particle[0].position[0] - particle[1].position[0]);
   float xVelVec = (particle[0].velocity[0] - particle[1].velocity[0]);

   float yPosVec = (particle[0].position[1] - particle[1].position[1]);
   float yVelVec = (particle[0].velocity[1] - particle[1].velocity[1]);

   float zPosVec = (particle[0].position[2] - particle[1].position[2]);
   float zVelVec = (particle[0].velocity[2] - particle[1].velocity[2]);

   float gravity = particle[1].mass * 0.98;

   float restDistanceX = 0;
   float restDistanceY = 7	;
   float restDistanceZ = 0;

   float magnitude = sqrt(pow(xPosVec, 2) + pow(yPosVec, 2) + pow(zPosVec, 2));

   particle[0].force[0] = -(springConstant * (magnitude - restDistanceX) + dampConstant * xVelVec * xPosVec/magnitude) * xPosVec/magnitude;

   particle[0].force[1] = -(springConstant * (magnitude - restDistanceY) + dampConstant * yVelVec * yPosVec/magnitude) * yPosVec/magnitude - gravity;

   particle[0].force[2] = -(springConstant * (magnitude - restDistanceZ) + dampConstant * zVelVec * zPosVec/magnitude) * zPosVec/magnitude;
}

int ParticleSystem::ParticleDerivative() {
   int index, destIndex = 0;
   Clear_Forces();
   Compute_Forces();
   for(index = 0; index < numP; index++) {
      temp1[destIndex++] = particle[index].velocity[0];
      temp1[destIndex++] = particle[index].velocity[1];
      temp1[destIndex++] = particle[index].velocity[2];
      temp1[destIndex++] = particle[index].force[0]/particle[index].mass;
      temp1[destIndex++] = particle[index].force[1]/particle[index].mass;	
      temp1[destIndex++] = particle[index].force[2]/particle[index].mass;		
   }
}

void ParticleSystem::ScaleVector(float deltaT) {
   for(int index = 0; index < 6 * numP; index++ ) {
      temp1[index] *= deltaT;
   }
}

void ParticleSystem::AddVectors() {
   for(int index = 0; index < 6 * numP; index++) {
      temp2[index] += temp1[index];
   }
}

void ParticleSystem::EulerStep(float deltaT) {
   
   ParticleDerivative();
   ScaleVector(deltaT);
   ParticleGetState();
   AddVectors();
   ParticleSetState();
   time += deltaT;
}

using namespace std;

//position and color data handles
GLuint triBuffObj, colBuffObj;
GLint h_lightColor;
//flag and ID to toggle on and off the shader
int shade = 1;
int ShadeProg;	
GLuint ibo;
GLuint positions, uvs, normals;
float g_transX = 0, g_transY = 6;
ParticleSystem particleSystem = ParticleSystem::ParticleSystem(2, 10, 5, 5);
//Handles to the shader data
GLint h_aPosition;
GLint h_uColor, h_aUV;
GLint h_uModelMatrix;
GLint h_uViewMatrix;
GLint h_uProjMatrix;
int g_startx, g_starty, g_endx, g_endy;
GLuint CubeBuffObj, CIndxBuffObj, GrndBuffObj, GIndxBuffObj, normalBuffObj, uvBuffObj, matBuffObj, GNBuffObj;
int g_CiboLen, g_GiboLen;
float yPos = 0, xPos = 0, zPos = 0;
static float g_width, g_height;
float g_trans = -20;
float g_Mtrans = 0;
float g_angle = 0;
static float g_scale = 1;
float scale = 0.05;
float radius  = 5;
static const float g_groundY = -2.0;      // y coordinate of the ground
static const float g_groundSize = 20.0;   // half the ground length
int vertexCount;
GLint h_uLightPos;
GLint h_uLightColor, h_aNormal;
int TriangleCount;
GLint h_uMatAmb, h_uMatDif, h_uMatSpec, h_uMatShine;
glm::vec3 Up = glm::vec3(0.0, 1.0, 0.0);
glm::vec3 Eye;
glm::vec3 lookAtPtr;
glm::vec3 lookV;
glm::vec3 w;
glm::vec3 u;
glm::vec3 v;
int check = -1;
float eyeX = 0, eyeY = 0, eyeZ = 15.0;
float lookX, lookY, lookZ;
float phi = -M_PI/2, theta = 0;
int g_view_mode = 0, g_string_mode = 0;
/* projection matrix  - do not change */
void SetProjectionMatrix() {
  glm::mat4 Projection = glm::perspective(80.0f, (float)g_width/g_height, 0.1f, 100.f);	
  safe_glUniformMatrix4fv(h_uProjMatrix, glm::value_ptr(Projection));
}
/* camera controls - do not change */
void SetView() {

  if(theta >= .5) {
      theta = .5;
  }
  if(theta <= -.5)
      theta = -.5;

  lookX = radius * glm::cos(theta) * glm::cos(phi);
  lookY = radius * glm::sin(theta);
  lookZ = radius * glm::cos(theta) * glm::cos(M_PI/2 - phi);

  Eye = glm::vec3(eyeX, eyeY, eyeZ);
  lookAtPtr = glm::vec3(lookX, lookY, lookZ) + Eye;

  lookV = lookAtPtr - Eye;

  w = glm::normalize(-lookV);
  u = glm::normalize(glm::cross(Up, w));
  v = w * u;
  printf("w %.1f %.1f %.1f\n", w.x, w.y, w.z);
  printf("u %.1f %.1f %.1f\n", u.x, u.y, u.z);
  printf("Up %.1f %.1f %.1f\n", Up.x, Up.y, Up.z);
if(check == 1){
      eyeX += -w.x/10.0;
      eyeY += -w.y/10.0;
      eyeZ += -w.z/10.0;
      
      lookX += -w.x/10.0;
      lookY += -w.y/10.0;
      lookZ += -w.z/10.0;
}
else if(check == 2){
      eyeX -= -w.x/10.0;
      eyeY -= -w.y/10.0;
      eyeZ -= -w.z/10.0;
      
      lookX -= -w.x/10.0;
      lookY -= -w.y/10.0;
      lookZ -= -w.z/10.0;
}
else if(check == 3){
      eyeX += u.x/10.0;
      eyeY += u.y/10.0;
      eyeZ += u.z/10.0;
      
      lookX += u.x/10.0;
      lookY += u.y/10.0;
      lookZ += u.z/10.0;
}
else if(check == 4){
      eyeX -= u.x/10.0;
      eyeY -= u.y/10.0;
      eyeZ -= u.z/10.0;
      
      lookX -= u.x/10.0;
      lookY -= u.y/10.0;
      lookZ -= u.z/10.0;
}
  check = 0;

  printf("lookX %.1f %.1f %.1f\n", lookX, lookY, lookZ);
  printf("eye %.1f %.1f %.1f\n", eyeX, eyeY, eyeZ);

  glm::mat4 RotateX = glm::lookAt(Eye, lookAtPtr, Up); 
  safe_glUniformMatrix4fv(h_uViewMatrix, glm::value_ptr(RotateX));
}

/* model transforms - change these to create a shape with boxes*/
void SetModel(float xPos, float yPos, float zPos) {
  glm::mat4 Trans = glm::translate( glm::mat4(1.0f), glm::vec3(xPos, yPos, zPos));
  Trans = glm::scale(Trans, glm::vec3(scale, scale, scale));
  safe_glUniformMatrix4fv(h_uModelMatrix, glm::value_ptr(Trans));
}
void SetModelI() {
  glm::mat4 tmp = glm::translate(glm::mat4(1.0f), glm::vec3(0.0, 0.0, 0.0));
  safe_glUniformMatrix4fv(h_uModelMatrix, glm::value_ptr(tmp));
}

static void initGround() {

  // A x-z plane at y = g_groundY of dimension [-g_groundSize, g_groundSize]^2
    float GrndPos[] = { 
    -g_groundSize, -g_groundSize, -g_groundSize, 
    -g_groundSize, -g_groundSize,  g_groundSize, 
     g_groundSize, -g_groundSize,  g_groundSize, 
     g_groundSize, -g_groundSize, -g_groundSize,
    -g_groundSize,  g_groundSize, -g_groundSize, 
    -g_groundSize,  g_groundSize,  g_groundSize, 
     g_groundSize,  g_groundSize,  g_groundSize, 
     g_groundSize,  g_groundSize, -g_groundSize,
    -g_groundSize, -g_groundSize,  g_groundSize, 
    -g_groundSize,  g_groundSize,  g_groundSize, 
     g_groundSize,  g_groundSize,  g_groundSize, 
     g_groundSize, -g_groundSize,  g_groundSize,
    -g_groundSize, -g_groundSize, -g_groundSize, 
    -g_groundSize,  g_groundSize, -g_groundSize, 
     g_groundSize,  g_groundSize, -g_groundSize, 
     g_groundSize, -g_groundSize, -g_groundSize,
    -g_groundSize, -g_groundSize, -g_groundSize, 
    -g_groundSize,  g_groundSize, -g_groundSize, 
    -g_groundSize,  g_groundSize,  g_groundSize, 
    -g_groundSize, -g_groundSize,  g_groundSize,
     g_groundSize, -g_groundSize, -g_groundSize, 
     g_groundSize,  g_groundSize, -g_groundSize, 
     g_groundSize,  g_groundSize,  g_groundSize, 
     g_groundSize, -g_groundSize,  g_groundSize
    };

    float GrndNorm[] = { 
     0, 1, 0, 
     0, 1, 0, 
     0, 1, 0, 
     0, 1, 0, 
     0, 1, 0, 
     0, 1, 0,
     0, -1, 0, 
     0, -1, 0, 
     0, -1, 0, 
     0, -1, 0, 
     0, -1, 0, 
     0, -1, 0,
     0, 0, -1, 
     0, 0, -1, 
     0, 0, -1, 
     0, 0, -1, 
     0, 0, -1, 
     0, 0, -1,
     0, 0, 1, 
     0, 0, 1, 
     0, 0, 1, 
     0, 0, 1, 
     0, 0, 1, 
     0, 0, 1,
     1, 0, 0, 
     1, 0, 0, 
     1, 0, 0, 
     1, 0, 0, 
     1, 0, 0, 
     1, 0, 0,
     -1, 0, 0, 
     -1, 0, 0, 
     -1, 0, 0, 
     -1, 0, 0, 
     -1, 0, 0, 
     -1, 0, 0
    };

    unsigned short idx[] = {0, 1, 2, 0, 2, 3, 4, 5, 6, 4, 6, 7, 8, 9, 10, 8, 10, 11, 12, 13, 14, 12, 14, 15, 16, 17, 18, 16, 18, 19, 20, 21, 22, 20, 22, 23};
    
    g_GiboLen = 36;
    glGenBuffers(1, &GrndBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, GrndBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(GrndPos), GrndPos, GL_STATIC_DRAW);

    glGenBuffers(1, &GIndxBuffObj);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, GIndxBuffObj);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(idx), idx, GL_STATIC_DRAW);
    
    glGenBuffers(1, &GNBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, GNBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(GrndNorm), GrndNorm, GL_STATIC_DRAW);

}

void SetMaterial(int input) {
   glUseProgram(ShadeProg);
   if(input == 0) {
	glUniform3f(h_uMatAmb, 0.06, 0.06, 0.2);
	glUniform3f(h_uMatDif, 0.8, 0.8, 0.0);
	glUniform3f(h_uMatSpec, 0.8, 0.8, 0.1);
	glUniform1f(h_uMatShine, 20.0);
   }
   else if(input == 1) {
	glUniform3f(h_uMatAmb, 0.2, 0.2, 0.5);
	glUniform3f(h_uMatDif, 0.0, 0.08, 0.5);
	glUniform3f(h_uMatSpec, 0.4, 0.4, 0.2);
	glUniform1f(h_uMatShine, 200.0);
   }
}

static void initSpring() {
  float CubePos[] = {
    -0.5, -0.5, -0.5, 
    -0.5, 0.5, -0.5, 
    0.5, 0.5, -0.5, 
    0.5, -0.5, -0.5, 
    -0.5, -0.5, 0.5, 
    -0.5, 0.5, 0.5, 
    0.5, 0.5, 0.5, 
    0.5, -0.5, 0.5
  };
  
   unsigned short idx[] = {0, 1, 2, 0, 2, 3, 7, 6, 4, 4, 6, 5, 1, 5, 6, 1, 6, 2, 0, 3, 7, 0, 7, 4};

    g_CiboLen = 24;
    glGenBuffers(1, &CubeBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, CubeBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(CubePos), CubePos, GL_STATIC_DRAW);

    glGenBuffers(1, &CIndxBuffObj);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, CIndxBuffObj);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(idx), idx, GL_STATIC_DRAW);
}


/* Initialize the geometry */
void InitGeom() {
    initGround();
    std::ifstream modelFile("beachball.obj");
    // parse the meshes from the file
    Model model;
    model.load(modelFile);
    // create VBOs from the first mesh in the file.
    // Any of the parameters can be NULL and the corresponding VBO won't be created.
    vertexCount = model.meshes()[0].makeVBO(&ibo, &positions, &uvs, &normals);
}

/*function to help load the shaders (both vertex and fragment */
/* for this assignment we are doing anything interesting with the fragment sahder */
int InstallShader(const GLchar *vShaderName, const GLchar *fShaderName) {
   GLuint VS; //handles to shader object
   GLuint FS; //handles to frag shader object
   GLint vCompiled, fCompiled, linked; //status of shader

   VS = glCreateShader(GL_VERTEX_SHADER);
   FS = glCreateShader(GL_FRAGMENT_SHADER);

   //load the source
   glShaderSource(VS, 1, &vShaderName, NULL);
   glShaderSource(FS, 1, &fShaderName, NULL);

   //compile shader and print log
   glCompileShader(VS);
   /* check shader status requires helper functions */
   printOpenGLError();
   glGetShaderiv(VS, GL_COMPILE_STATUS, &vCompiled);
   printShaderInfoLog(VS);

   //compile shader and print log
   glCompileShader(FS);
   /* check shader status requires helper functions */
   printOpenGLError();
   glGetShaderiv(FS, GL_COMPILE_STATUS, &fCompiled);
   printShaderInfoLog(FS);

   if (!vCompiled || !fCompiled) {
       printf("Error compiling either shader %s or %s", vShaderName, fShaderName);
      return 0;
   }

   //create a program object and attach the compiled shader
   ShadeProg = glCreateProgram();
   glAttachShader(ShadeProg, VS);
   glAttachShader(ShadeProg, FS);

   glLinkProgram(ShadeProg);
   /* check shader status requires helper functions */
   printOpenGLError();
   glGetProgramiv(ShadeProg, GL_LINK_STATUS, &linked);
   printProgramInfoLog(ShadeProg);

   glUseProgram(ShadeProg);

   /* get handles to attribute and uniform data in shader */
   h_aNormal = safe_glGetAttribLocation(ShadeProg, "aNormal");
   h_lightColor = safe_glGetUniformLocation(ShadeProg,  "lightColor");
   h_aPosition = safe_glGetAttribLocation(ShadeProg, "aPosition");
   h_uProjMatrix = safe_glGetUniformLocation(ShadeProg, "uProjMatrix");
   h_uViewMatrix = safe_glGetUniformLocation(ShadeProg, "uViewMatrix");
   h_uModelMatrix = safe_glGetUniformLocation(ShadeProg, "uModelMatrix");
   h_uLightPos = safe_glGetUniformLocation(ShadeProg, "lightPosition");
   h_uMatAmb = safe_glGetUniformLocation(ShadeProg, "uMat.aColor");
   h_uMatDif = safe_glGetUniformLocation(ShadeProg, "uMat.dColor");
   h_uMatSpec = safe_glGetUniformLocation(ShadeProg, "uMat.sColor");
   h_uMatShine = safe_glGetUniformLocation(ShadeProg, "uMat.shine");

   printf("sucessfully installed shader %d\n", ShadeProg);
   return 1;
}


/* Some OpenGL initialization */
void Initialize ()					// Any GL Init Code 
{
	// Start Of User Initialization
	glClearColor (1.0f, 1.0f, 1.0f, 1.0f);								
	// Black Background
 	glClearDepth (1.0f);	// Depth Buffer Setup
 	glDepthFunc (GL_LEQUAL);	// The Type Of Depth Testing
	glEnable (GL_DEPTH_TEST);// Enable Depth Testing
}

/* Main display function */
void Draw (void)
{
   glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);				
   glUseProgram(ShadeProg);

   SetProjectionMatrix();
   SetView();


   glUniform3f(h_lightColor, 1.0, 1.0, 1.0);
   glUniform3f(h_uLightPos, 0.0, 18.0, 0.0);

   SetModelI();
   SetMaterial(0);
   safe_glEnableVertexAttribArray(h_aPosition);
   glBindBuffer(GL_ARRAY_BUFFER, GrndBuffObj);
   safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);

   safe_glEnableVertexAttribArray(h_aNormal);
   glBindBuffer(GL_ARRAY_BUFFER, GNBuffObj);
   safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);

   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, GIndxBuffObj);
   glDrawElements(GL_TRIANGLES, g_GiboLen, GL_UNSIGNED_SHORT, 0);

   safe_glDisableVertexAttribArray(h_aPosition);
   safe_glDisableVertexAttribArray(h_aNormal);


   SetModel(8 * particleSystem.particle[0].position[0], 8 * particleSystem.particle[0].position[1], zPos);
   SetMaterial(1);
   glEnableVertexAttribArray(h_aPosition);
   glBindBuffer(GL_ARRAY_BUFFER, positions);
   glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);

   glEnableVertexAttribArray(h_aNormal);
   glBindBuffer(GL_ARRAY_BUFFER, normals);
   glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);

   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, ibo);
   glDrawElements(GL_TRIANGLES, vertexCount, GL_UNSIGNED_INT, 0);
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

   safe_glDisableVertexAttribArray(h_aPosition);
   safe_glDisableVertexAttribArray(h_aNormal);

   glUseProgram(0);	
   glutSwapBuffers();

}

/* Reshape - note no scaling as perspective viewing*/
void ReshapeGL (int width, int height)								
{
	g_width = (float)width;
	g_height = (float)height;
	glViewport (0, 0, (GLsizei)(width), (GLsizei)(height));				

}

//the keyboard callback
void keyboard(unsigned char key, int x, int y ){
  switch( key ) {
    case 'w':
check = 1;

      break;
    case 's':
check = 2;

      break;
    case 'a':
check = 4;

      break;
    case 'd':
check = 3;
    case 'm':
      g_Mtrans -= .1;
      break;
    case 'n':
      g_Mtrans += .1;
      break;
    case '+':
      if(particleSystem.particle[0].mass >= 100) {
         printf("Mass: %.1f. Can't increase the mass anymore.\n", particleSystem.particle[0].mass);
      }
      else {
         scale += 0.0025;
         particleSystem.particle[0].mass += 2.0;
      }
      printf("Mass: %.1f\n", particleSystem.particle[0].mass);
      break;
    case '-':
      if(particleSystem.particle[0].mass <= 3.0) {
         printf("Mass: %.1f. Can't decrease the mass anymore.\n", particleSystem.particle[0].mass);
      }
      else {
         scale -= 0.0025;
         particleSystem.particle[0].mass -= 2.0;
      }
      printf("Mass: %.1f\n", particleSystem.particle[0].mass);
      break;
    case 'q': case 'Q' :
      exit( EXIT_SUCCESS );
      break;
  }
  glutPostRedisplay();
}

void idle(void) {
   particleSystem.EulerStep(0.05);
   xPos = particleSystem.particle[0].position[0];
   yPos = particleSystem.particle[0].position[1];
   zPos = particleSystem.particle[0].position[2];

   particleSystem.printParticle();
 
   glutPostRedisplay();
}

/*2D coord transforms - slightly off, but fine for user interaction for p2 */
float p2i_x(int p_x) {
  float x_i = ( (float)p_x - ((g_width-1.0)/2.0) )*2.0/g_width;
  return(x_i);
}

float p2i_y(int p_y) {
  return( ( (float)p_y - ((g_height-1.0)/2.0) )*2.0/g_height);
}

float p2w_x(int p_x) {
  float x_i = ( (float)p_x - ((g_width-1.0)/2.0) )*2.0/g_width;
  return(((float)g_width/(float)g_height)*x_i);
}

float p2w_y(int p_y) {
  return( ( (float)p_y - ((g_height-1.0)/2.0) )*2.0/g_height);
}

void mouse(int button, int state, int x, int y) {
   if (state == GLUT_DOWN) {
      g_startx = x;
      g_starty = g_height-y-1;
      if (button == GLUT_LEFT_BUTTON) {
         g_string_mode = 1;
         glutIdleFunc(NULL);
      } else if(button == GLUT_RIGHT_BUTTON) {
         g_view_mode = 1;
      }
   } else if (state == GLUT_UP) {
	glutIdleFunc(idle);
        g_view_mode = g_string_mode = 0;
   }
}
void mouseMove(int x, int y) {
  g_endx = x;
  g_endy = g_height-y-1;
    
  float startWX = p2w_x(g_startx);
  float startWY = p2w_y(g_starty);
  float endWX = p2w_x(g_endx);
  float endWY = p2w_y(g_endy);

  xPos += endWX - startWX;
  yPos += endWY - startWY;
  
  if(g_string_mode == 1) {
     particleSystem.particle[0].position[0] = xPos;
     particleSystem.particle[0].position[1] = yPos;
     particleSystem.particle[0].velocity[0] = 0; 
     particleSystem.particle[0].velocity[1] = 0;   
  }
  if(g_view_mode == 1) {
     theta += (-endWY + startWY) * M_PI / 5;
     phi += (-endWX + startWX) * M_PI / 5;
  }
  g_startx = g_endx;
  g_starty = g_endy;

  glutPostRedisplay();
}


int main( int argc, char *argv[] )
{
   glutInit( &argc, argv );
   glutInitWindowPosition( 20, 20 );
   glutInitWindowSize( 400, 400 );
   glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
   glutCreateWindow("My Fragment shader");
   glutReshapeFunc( ReshapeGL );
   glutDisplayFunc( Draw );
   glutKeyboardFunc( keyboard );
   glutMouseFunc(mouse);
   glutMotionFunc(mouseMove);
	glutIdleFunc(idle);
   Initialize();
	
	//test the openGL version
	getGLversion();
	//install the shader
	if (!InstallShader(textFileRead((char *)"Lab4_vert.glsl"), textFileRead((char *)"Lab4_frag.glsl"))) {
		printf("Error installing shader!\n");
		return 0;
	}
		
	InitGeom();
  	glutMainLoop();
   return 0;
}
