struct Material {
  vec3 aColor;
  vec3 dColor;
  vec3 sColor;
  float shine;
};

attribute vec3 aPosition;
attribute vec3 aNormal;
uniform int uFlag;

uniform vec3 lightColor;
uniform vec3 lightPosition;
uniform mat4 uProjMatrix;
uniform mat4 uViewMatrix;
uniform mat4 uModelMatrix;

uniform vec3 spec, diff, ambi, shine;

varying vec3 vColor;

varying vec3 PhongNormal;
varying vec3 lightVector;
varying vec3 PhongLightCol;
varying vec3 PhongView;
varying vec3 PhongReflextion;

uniform Material uMat;

void main() {
  vec4 vPosition;
  vec3 light;
  vec3 Spec, Diffuse ;
  vec3 Ambi;
  
  /* First model transforms */
  vPosition = uModelMatrix* vec4(aPosition.x, aPosition.y, aPosition.z, 1);
  light = normalize(lightPosition - vec3(vPosition.x, vPosition.y, vPosition.z));
  vPosition = uViewMatrix * vPosition;
  gl_Position = uProjMatrix*vPosition;
  
  vec3 N = (uModelMatrix * vec4(aNormal, 0)).xyz;
  vec3 V = vec3(-vPosition.x, -vPosition.y, -vPosition.z);
  vec3 Refl = -light + 2.0 * (dot(normalize(light), normalize(N))) * N;
  Refl = reflect(-light, normalize(N));
  Refl = normalize(Refl);
  PhongNormal = N;
  PhongReflextion = Refl;
  PhongView = V;
  PhongLightCol = lightColor;
 
  lightVector = light;

  Spec = lightColor * uMat.sColor * pow(clamp(dot(normalize(Refl), normalize(V)), 0.0, 1.0 ),uMat.shine);
  Diffuse = lightColor * uMat.dColor * clamp(max(dot(normalize(N) , (normalize(light))), 0.0), 0.0, 1.0);
  Ambi = lightColor * uMat.aColor;

  vColor = N;
}

